//******************************************
//Exemple Pompe de code pour l'utilisation de la classe Stat
//de la librarie Stat
//Basé sur le diagram d'état réalisé par Olivier Le Gallo
//Codage Arnaud Dugas
//******************************************



#include <Arduino.h>
#include "obj_stat.h"  //Si obj_stat.h et obj_stat.cpp sont dans le répertoire local
//#include <obj_stat.h> //Si obj_stat.h et obj_stat.cpp sont dans le répertoire obj_stat
//du répertoire libraries

//******************************************
//Hardware Carte Mega2560
//******************************************
#define B_AU A2   //Câblage d’un bouton poussoir sur la pin A2
#define B_BPM 2   //Câblage d’un bouton poussoir sur la pin 2
#define B_BPA 4   //Câblage d’un bouton poussoir sur la pin 4
#define B_DPP 6   //Câblage d’un bouton poussoir sur la pin 6
#define B_HM 8    //Câblage d’un bouton poussoir sur la pin 8
#define B_ACQ 12  //Câblage d’un bouton poussoir sur la pin 12
#define B_DHM A4  //Câblage d’un bouton poussoir sur la pin A4
#define L_PM A0   //Câblage d’une led sur la pin A0 (Pour par exemple l'état de la pompe (Marche ou Arret)

//******************************************
//Déclaration des objets
//******************************************

//Entrées
Event AU("AU");    //bouton poussoire
Event BPM("BPM");  //bouton poussoire
Event BPA("BPA");  //bouton poussoire
Event DPP("DPP");  //bouton poussoire
Event HM("HM");    //bouton poussoire
Event Acq("Acq");  //bouton poussoire
Event DHM("DHM");  //bouton poussoire

//Sorties
DigitalOut Pompe("Pompe");  //led

//Déclaration des état
Stat Pompe_a_l_arret("Pompe à l'arret", 0);      //id = 0
Stat Pompe_en_marche("Pompe en marche", 1);      //id = 1
Stat Defaut("Défaut", 3);                        //id = 3
Stat Remise_en_service("Remise en service", 4);  //id = 4
//Sous états de l'état id=1
Stat Attente_liberation_de_HM("Attente libération de HM", 11);  //id = 11
Stat Attente_HM("Attente libération de HM", 12);                //id = 12
//Sous états de l'état id=11
Stat Mise_en_marche_de_la_pompe("Mise en marche de la pompe", 111);  //id = 111
Stat Attente_de_40s_maxi("Attente de 40s maxi", 112);                //id = 112
Stat Attente_de_20s_maxi("Attente de 20s maxi", 113);                //id = 113

//Pointeur vers l'état actif
Stat* pActif;       //Pointeur sur l'état actif du niveau 0
Stat* pActif_id1;   //Pointeur sur l'état actif du niveau id = 1
Stat* pActif_id11;  //Pointeur sur l'état actif du niveau id = 11

//******************************************
//Setup
//******************************************
void setup() {
  //******************************************
  //Setup hardware
  //******************************************
  AU.setPin(B_AU);     //Affectation de la pin à l'objet
  BPM.setPin(B_BPM);   //Affectation de la pin à l'objet
  BPA.setPin(B_BPA);   //Affectation de la pin à l'objet
  DPP.setPin(B_DPP);   //Affectation de la pin à l'objet
  HM.setPin(B_HM);     //Affectation de la pin à l'objet
  Acq.setPin(B_ACQ);   //Affectation de la pin à l'objet
  DHM.setPin(B_DHM);   //Affectation de la pin à l'objet
  Pompe.setPin(L_PM);  //Affectation de la pin à l'objet


  //******************************************
  //Port série
  //******************************************
  Serial.begin(9600);
  //******************************************
  //Activation des log
  //******************************************
  Pompe.log = true;
  AU.log = true;
  HM.log = true;
  BPM.log = true;
  Acq.log = true;

  //******************************************
  //Mise à jour des entrées sorties
  //******************************************
  //Entrées
  AU.update();
  BPM.update();
  BPA.update();
  DPP.update();
  HM.update();
  Acq.update();
  DHM.update();
  Pompe.update();

  //******************************************
  //Etat initial
  //******************************************
  pActif = Pompe_a_l_arret.activate();
  pActif_id1 = 0;
  pActif_id11 = 0;
}

//******************************************
//Loop
//******************************************
void loop() {
  //#############################
  //Description de l'évolution
  //#############################
  //Description de l'évolution niveau 0
  transition(pActif, Pompe_en_marche, Pompe_a_l_arret, AU.rising());
  transition(pActif, Pompe_en_marche, Pompe_a_l_arret, BPA.rising());
  transition(pActif, Pompe_en_marche, Pompe_a_l_arret, DPP.rising());
  if (transition(pActif_id11, Pompe_a_l_arret, Mise_en_marche_de_la_pompe, BPM.rising(), (HM.stat() && (!AU.stat()) && (!DHM.stat()) && (!DPP.stat())))) {
    //Cette transition provoque aussi l'activation des états :
    // Attente_liberation_de_HM et Pompe_en_marche
    pActif = Pompe_en_marche.activate();
    pActif_id1 = Attente_liberation_de_HM.activate();
  }

  if (after(pActif, Attente_de_20s_maxi, Defaut, 20000)) {  //transition après 20000 ms
    //Cette transition provoque aussi la désactivation des états :
    // Attente_liberation_de_HM et Pompe_en_marche
    Pompe_en_marche.desactivate();
    Attente_liberation_de_HM.desactivate();
    pActif_id1 = 0;
    pActif_id11 = 0;
  }
  if (after(pActif, Attente_HM, Defaut, 5000)) {  //transition après 5000 ms
    //Cette transition provoque aussi la désactivation de l'état :
    // Pompe_en_marche
    Pompe_en_marche.desactivate();
    pActif_id1 = 0;
  }
  transition(pActif, Defaut, Remise_en_service, Acq.rising());
  transition(pActif, Remise_en_service, Pompe_a_l_arret, true);
  //Description de l'évolution niveau 1
  if (transition(pActif_id1, Attente_liberation_de_HM, Attente_HM, HM.falling())) {
    //Cette transition provoque aussi la désactivation de l'état :
    // Mise_en_marche_de_la_pompe ou Attente_de_40s_maxi ou Attente_de_20s_maxi
    //On utilise le pointeur pActif_id11 pour savoir quel état est actif pour le désactiver
    pActif_id11->desactivate();
    pActif_id11 = 0;
  }
  if (transition(pActif_id11, Attente_HM, Attente_de_40s_maxi, HM.rising())) {
    //Cette transition provoque aussi l'activation de l'état :
    // Attente_liberation_de_HM
    pActif_id1 = Attente_liberation_de_HM.activate();
  }
  //Description de l'évolution niveau 2
  transition(pActif_id11, Mise_en_marche_de_la_pompe, Attente_de_40s_maxi, true);
  after(pActif_id11, Attente_de_40s_maxi, Attente_de_20s_maxi, 40000);

  //#############################
  //Activités
  //#############################
  //exit (à placer avant les entry)
  //do
  //entry
  if (Mise_en_marche_de_la_pompe.entry_()) {
    Pompe.on();  //Mise en marche effective de la pompe
  }
  if (Defaut.entry_()) {
    Pompe.off();  //Mise à l'arret effective de la pompe
  }
  //#############################
  //Mise à jour
  //#############################
  //Entrées
  AU.update();
  BPM.update();
  BPA.update();
  DPP.update();
  HM.update();
  Acq.update();
  DHM.update();
  Pompe.update();

  //Sorties
  Pompe.update();

  //état
  Pompe_a_l_arret.update();
  Pompe_en_marche.update();
  Defaut.update();
  Remise_en_service.update();
  //Sous états de l'état id=1
  Attente_liberation_de_HM.update();
  Attente_HM.update();
  //Sous états de l'état id=11
  Mise_en_marche_de_la_pompe.update();
  Attente_de_40s_maxi.update();
  Attente_de_20s_maxi.update();
}