#programme Homme mort de Robert à partir du stm fourni
#copie du fichier homme_mort.py sur la pyboard (comme sur une clé USB)
#execution dans le shell de la pyboard : exec(open('homme_mort.py').read())
#voir rubrique système puis micropython sur le site upsti

#pour des raisons de simplicité de programmation je numérote les états
#Pompe à l'arrêt : etat=1
#Pompe en marche : etat_composite2
#Attente libération IHM : etat composite2
#Mise en marche de la pompe : etat=4
#Attente de 40s maxi : etat=5
#Attente de 20s maxi : etat=6
#Attente HM : etat=7
#Défaut : etat=8
#Remise en service : etat=9

#la création du programme suivant est uniquement dû à la lecture du stm

######## importation des bibliothèques
import pyb #importation de la bibliothèque de la pyboard

######### déclaration entrées
BPM=pyb.Pin(pyb.Pin.board.X1,pyb.Pin.IN,pyb.Pin.PULL_DOWN)#PULL DOWN limite les effets de rebonds
AU=pyb.Pin(pyb.Pin.board.X2,pyb.Pin.IN, pyb.Pin.PULL_DOWN) 
BPA=pyb.Pin(pyb.Pin.board.X3,pyb.Pin.IN, pyb.Pin.PULL_DOWN)
DDP=pyb.Pin(pyb.Pin.board.X4,pyb.Pin.IN, pyb.Pin.PULL_DOWN)
HM=pyb.Pin(pyb.Pin.board.X5,pyb.Pin.IN, pyb.Pin.PULL_DOWN)
Acq=pyb.Pin(pyb.Pin.board.X6,pyb.Pin.IN, pyb.Pin.PULL_DOWN)
fin_mise_service=pyb.Pin(pyb.Pin.board.X7,pyb.Pin.IN, pyb.Pin.PULL_DOWN) #dans le stm fourni pas d'info sur l'évènement de fin de mise en service (je l'ai donc placé comme une entrée)
DHM=pyb.Pin(pyb.Pin.board.X8,pyb.Pin.IN, pyb.Pin.PULL_DOWN)

######## déclaration sorties
Marche_pompe=pyb.Pin(pyb.Pin.board.Y1,pyb.Pin.OUT_PP)  #(le choix des Xi et des Yi est associé au câblage physique sur la pyboard)
Alarme1=pyb.Pin(pyb.Pin.board.Y2,pyb.Pin.OUT_PP)
Alarme2=pyb.Pin(pyb.Pin.board.Y3,pyb.Pin.OUT_PP)
Alarme3=pyb.Pin(pyb.Pin.board.Y4,pyb.Pin.OUT_PP)

######### initialisation constantes
#etatcomposite=0
etat=1 #etat associé au pseudo état initial
etat_composite2=0
etat_composite3=0
etr=0 #évènement temporel relatif

######## fonctions et interruptions (scrutations des évènements dits externes)
# procédures appellées par une interruption du front des entrées voir + bas

def detect_BPM(pin):
    global etat
    global etat_composite2
    global etat_composite3
    
    if etat==1 and HM.value()==True and AU.value()==False and DHM.value()==False and DDP.value()==False : #gestion garde
        etat_composite2=True
        etat_composite3=True
        etat=4
    return
    
def detect_AU(pin):
    global etat
    global etat_composite2
    global etat_composite3
    
    if etat_composite2==True:
        etat_composite2=False
        etat_composite3=False
        etat=1
    return
    
def detect_BPA(pin):
    global etat
    global etat_composite2
    global etat_composite3
    
    if etat_composite2==True:
        etat_composite2=False
        etat_composite3=False
        etat=1
    return
 
def detect_DDP(pin):
    global etat
    global etat_composite2
    global etat_composite3
    
    if etat_composite2==True:
        etat_composite2=False
        etat_composite3=False
        etat=1
    return

def detect_HM_Falling(pin):
    global etat
    global etat_composite3
    global etr
    if etat_composite3==True:
        etat=7
        etat_composite3=False
        etr=pyb.millisec()#lancement d'un chronomètre etr
    return
    
def detect_HM_Rising(pin):
    global etat
    global etat_composite2
    global etr
    
    if etat==7 and etat_composite2==True:
        etat=5
        etr=pyb.millisec() 
    return
 
def detect_Acq(pin):
    global etat
    if etat==8:
        etat=9
    return
    
def detect_Fin_mise_en_service(pin):
    global etat
    if etat==9:
        etat=1
    return

#création des interruptions associées aux évènements externes    
extint1 = pyb.ExtInt(BPM, pyb.ExtInt.IRQ_RISING, pyb.Pin.PULL_DOWN, detect_BPM)
extint2 = pyb.ExtInt(AU, pyb.ExtInt.IRQ_RISING, pyb.Pin.PULL_DOWN, detect_AU) 
extint3 = pyb.ExtInt(BPA, pyb.ExtInt.IRQ_RISING, pyb.Pin.PULL_DOWN, detect_BPA) 
extint4 = pyb.ExtInt(DDP, pyb.ExtInt.IRQ_RISING, pyb.Pin.PULL_DOWN, detect_DDP)
extint5 = pyb.ExtInt(HM, pyb.ExtInt.IRQ_RISING, pyb.Pin.PULL_DOWN, detect_HM_Rising)
extint6 = pyb.ExtInt(HM, pyb.ExtInt.IRQ_FALLING, pyb.Pin.PULL_DOWN, detect_HM_Falling)
extint7 = pyb.ExtInt(Acq, pyb.ExtInt.IRQ_RISING, pyb.Pin.PULL_DOWN, detect_Acq)
extint8 = pyb.ExtInt(fin_mise_service, pyb.ExtInt.IRQ_RISING, pyb.Pin.PULL_DOWN, detect_Fin_mise_en_service)


########### programme principal
while True:

#evolution graphe avec évènements internes
    if etat==5 and pyb.elapsed_millis(etr) > 40000 : #after(40s) situé après l'état5
        etat=6
        etr=pyb.millisec()

    if etat==6 and pyb.elapsed_millis(etr) > 20000 :
        etat=8
        etat_composite3=False
        etat_composite2=False
        
    if etat==7 and pyb.elapsed_millis(etr) > 5000:
        etat=8
        etat_composite2=False
    
#affectation des sorties
    if etat==1:
        Marche_pompe.low()
        Alarme1.low()
        Alarme2.low()
        Alarme3.low()
        
    if etat==4 :
        Marche_pompe.high()
        etat=5 #transition automatique après l'état 4
        etr=pyb.millisec()
        
    if etat==6:
        Alarme1.high()
        
    if etat==7:
        Alarme2.high()
        
    if etat==8:
        Marche_pompe.low()
        Alarme3.high()
