//******************************************
//Exemple de code pour l'utilisation de la classe Stat
//de la librarie Stat
//******************************************
//Cet exemple comporte 3 états :
// 	* un état LED_ON où la led est allumée,
// 	* un état LED_OFF où la led est éteinte,
// 	* un état LED_FLASH où la led clignote (500 ms).

//L’état LED_OFF est l’état initial.
//L’appuie sur le bouton poussoir, provoque le passage dans l’état LED_FLASH.
//5 secondes plus tard, on passe dans l’état LED_ON.
//Si entre temps, on appui sur le bouton, on repasse en LED_OFF.
//Lorsque l’on est en LED_ON, l’appui sur le bouton permet de repasser en LED_OFF.


#include <Arduino.h>
#include "obj_stat.h"  //Si obj_stat.h et obj_stat.cpp sont dans le répertoire local
//#include <obj_stat.h> //Si obj_stat.h et obj_stat.cpp sont dans le répertoire obj_stat
//du répertoire libraries

//******************************************
//Hardware
//******************************************
#define BOUTON 2  //Câblage d’un bouton poussoir sur la pin 2
#define LED A0    //Câblage d’une led sur la pin A0

//******************************************
//Déclaration des objets
//******************************************
Event bouton;    //bouton poussoire
DigitalOut led;  //led

Stat LED_OFF("Etat led off", 0);      //Etat LED_OFF, id = 0
Stat LED_FLASH("Etat led flash", 1);  //Etat LED_FLASH, id = 1
Stat LED_ON("Etat led on", 2);        //Etat LED_ON, id = 2

Stat* pActif;  //Pointeur sur l'état actif

//******************************************
//Setup
//******************************************
void setup() {
  //******************************************
  //Setup hardware
  //******************************************
  bouton.setPin(BOUTON);  //Affectation de la pin à l'objet
  led.setPin(LED);        //Affectation de la pin à l'objet
  //******************************************
  //Port série
  //******************************************
  Serial.begin(9600);
  //******************************************
  //Etat initial
  //******************************************
  pActif = LED_OFF.activate();
}

//******************************************
//Loop
//******************************************
void loop() {
  //Description de l'évolution
  transition(pActif, LED_OFF, LED_FLASH, bouton.rising());  //transition par appui sur le bouton
  after(pActif, LED_FLASH, LED_ON, 5000);                   //transition après 5000 ms
  transition(pActif, LED_FLASH, LED_OFF, bouton.rising());
  transition(pActif, LED_ON, LED_OFF, bouton.rising());

  //Activités
  //exit (à placer avant les entry)
  //do
  //entry
  if (LED_OFF.entry_()) {
    led.off();
  }
  if (LED_FLASH.entry_()) {
    led.flash(500);  //changement d'état tout les 500 ms
  }
  if (LED_ON.entry_()) {
    led.on();
  }
  //******************************************
  //Mise à jour
  //******************************************
  bouton.update();
  led.update();
  LED_OFF.update();
  LED_FLASH.update();
  LED_ON.update();
}