//******************************************
//Exemple de code pour l'utilisation de la classe Stat
//de la librarie Stat
//******************************************
//Cet exemple comporte 2 états :
// 	* un état LED_ON où la led est allumée,
// 	* un état LED_OFF où la led est éteinte,
//L’état LED_OFF est l’état initial.
//L’appuie sur le bouton poussoir, provoque le passage dans l’état LED_ON.
//Lors du passage de LED_OFF à LED_ON une deuxième led est allumée pendant 2 secondes.
//Lorsque l’on est en LED_ON, l’appui sur le bouton permet de repasser en LED_OFF
//à condition que la deuxième led soit éteinte (Condition de garde).
//En pratique, la première led ne peut donc pas être éteinte tant que le seconde est allumée.

#include <Arduino.h>
#include "obj_stat.h"  //Si obj_stat.h et obj_stat.cpp sont dans le répertoire local
//#include <obj_stat.h> //Si obj_stat.h et obj_stat.cpp sont dans le répertoire obj_stat
//du répertoire libraries

//******************************************
//Hardware
//******************************************
#define BOUTON 2  //Câblage d’un bouton poussoir sur la pin 2
#define LED A0    //Câblage d’une led sur la pin A0
#define LED2 A2   //Câblage d’une led sur la pin A2

//******************************************
//Déclaration des objets
//******************************************
Event bouton;     //bouton poussoire
DigitalOut led;   //led
DigitalOut led2;  //led

Stat LED_OFF("Etat led off", 0);  //Etat LED_OFF, id = 0
Stat LED_ON("Etat led on", 1);    //Etat LED_ON, id = 2

Stat* pActif;  //Pointeur sur l'état actif

//******************************************
//Setup
//******************************************
void setup() {
  //******************************************
  //Setup hardware
  //******************************************
  bouton.setPin(BOUTON);  //Affectation de la pin à l'objet
  led.setPin(LED);        //Affectation de la pin à l'objet
  led2.setPin(LED2);      //Affectation de la pin à l'objet
  //******************************************
  //Port série
  //******************************************
  Serial.begin(9600);
  //******************************************
  //Etat initial
  //******************************************
  pActif = LED_OFF.activate();
}

//******************************************
//Loop
//******************************************
void loop() {
  //Description de l'évolution
  if (transition(pActif, LED_OFF, LED_ON, bouton.rising())) {  //transition avec action
    led2.on(2000);                                             //on pendant 2 secondes
  }
  transition(pActif, LED_ON, LED_OFF, bouton.rising(), !led2.stat());  //transition avec condition de garde

  //Activités
  //exit (à placer avant les entry)
  //do
  //entry
  if (LED_OFF.entry_()) {
    led.off();
  }
  if (LED_ON.entry_()) {
    led.on();
  }
  //******************************************
  //Mise à jour
  //******************************************
  bouton.update();
  led.update();
  led2.update();
  LED_OFF.update();
  LED_ON.update();
}