////////////////////////////////////////////////////////
//Bibliothèque programmation par graph d'état (Stat Machine)
//Contient les objets :
//Stat, Timer, Event, Button, Pulse, DigitalOut, Bistable, EventFiltered
//Contient les fonctions :
//transition, when, after et Completion
//Version 2.0.1 du 17 11 2022,
//Auteur : Arnaud Dugas, CPGE - Lycée Livet, NANTES, dugas.arnaud@gmailcom
//Contribution : Quentin Quadrat, lecrapouille@gmail.com
////////////////////////////////////////////////////////

/*
MIT License

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

//Nouveauté version 1.3 : Important
//  La fonction stat() permet de savoir si l'état est actif. Attention, elle ne doit jamais être utilisée dans la transition immédiatement suivante au risque de rendre le systeme instable.
//  En effet, dans ce cas l'activité m_entry risque de ne jamais être exécutée (Cas ou dans l'activité m_entry on inititalise une variable utilisée dans la transition suivant).
//  La fonction _interne_stat() doit être utilisée à la place, _interne_stat() revoie true une fois m_entry exécutée.
//Nouveauté version 1.3.1 :
//  Ajout de la méthode resetStattime dans Cstat
//Nouveauté version 1.4.0 : Importante
//  Simplification de la class Cevent (On ne garde que .rising et .falling)
//  Ajout de la classe Cbutton qui reprend Cevent avec en plus :
//  .click, .doubleClick et .longPress
//  Refonte de la classe Ctimer (Suppression de Every)
//  Ajout de la classe Cpulse pour gérer le every
//  Correction de bug version 1.4.1 :
//  Correction de Cevent::SetPin et Cbutton::SetPin
//  Lecture de l'entrée pour mettre à jour l'état après la définition de la pin d'entrée
//Nouveauté version 1.4.3
//  Modification de la méthode update de CStat
//  Modification de la méthode update de CEvent
//  Modification de la méthode update de CButton
//  Optimisation de Cbutton::update
//Nouveauté version 1.4.4 du 31/05/2018
//  Correction bug affichage dans Serial sur CButton
//Nouveauté version 1.4.5 du 29/05/2019
//  Modification de la méthode update de CEvent et CButton
//  ANTI_REBOUNCE_TIME devient la valeur par défaut qui peut être modifié avec la propriété : anti_rebounce_time (CEvent et CButton)
//  Modification de la méthode constructeur de CButton
//  Le long_press_time peut être défini en début de programme par LONG_PRESS_TIME
//Nouveauté version 1.5.0 du 05/05/2020
//  Nouvelle fonctionalité
//  Ajout de la fonction when pour les transitions booleenes
//Nouveauté version 1.5.1 du 04/06/2020
//  Nouvelle fonctionalité
//  Ajout de la fonction completion pour les transition implicites
//  Ajout des log sur les transitions (activé par défaut)
//Nouveauté version 1.5.2 du 22/06/2020
//  Correction de bug	: Modification gestion résistance de PULL_UP (la version précédente ne fonctionnait pas sur Nano) Classe CEvent et CButton
//  Ajout : Cbutton::longPressStat() renvoie true pendant toute la durée de l'appui long
//  Ajout : Cbutton::longPressFalling() renvoie true sur le front descendant d'un appui long
//Nouveauté version 1.5.3 du 03/11/2020
//  Ajout : Cevent::get_stat_time() renvoie la date du dernier changement d'état (millis())
//  Ajout : Cbutton::get_stat_time() renvoie la date du dernier changement d'état (millis())
//  Modification : CdigitalOut::change() renvoie un true si il y a eu une action on ou off sur le cycle
//Nouveauté version 1.5.4 du 16/04/2021
//  Modifications : CdigitalOut et Cbistable
//  Il est maintenant possible de gérer les relais inversés (relais au niveau haut quand ils sont ouverts) (méthode setPin)
//  Modifications : Cevent et Cbutton
//  Il est maintenant possible d'inverser la lecture des entrées (méthode setPin)
//Nouveauté version 1.5.5 du 05/05/2021
//  Ajout : Cstat::get_stat_time() renvoie la date de la dernière activation ou du dernier reset
//  Correction de bug : Cbistable::stat2()
//  Ajout : Condition de garde possible sur le when
//Nouveauté version 1.6.0 du 28/05/2021
//  Réécriture de CButton comme héritère explicite de CEvent
//Nouveauté version 1.6.1 du 04/06/2021
//  Ajout de la class CEventFiltered comme héritère  de CEvent
//  Optimisation de Cevent, CeventFiltering, Cbutton
//Nouveauté version 1.6.2 du 21/06/2021
//  Ajout de CdigitalOut::toggle()
//Nouveauté version 1.6.3 du 27/08/2021
//  Modification de la méthode update de CdigitalOut
//  Correction comportement CdigitalOut::change() via update
//  Ajout de CdigitalOut::rising()
//  Ajout de CdigitalOut::falling()
//  Ajout de CdigitalOut::end_of_timer()
//Nouvauté version 2.0.0 du 24/10/2022
//  Remplacement par cohérence de get_stat_time par getStattime
//  Remplacement par cohérence de end_of_timer par endOfTimer
//  Correction fonctions transition, after, when, completion
//  CHANGEMENT DE TOUS LES NOM DE CLASS
//  CStat devient Stat, Cevent devient Event...
//Nouvauté version 2.0.1 du 17/11/2022
//  Correction de bug pour Arduino Nano 33 BLE
////////////////////////////////////////////////////////


#ifndef obj_stat_h
#define obj_stat_h

#include <Arduino.h>

//Class CButton et Cevent
#define ANTI_REBOUNCE_TIME 10  //Valeur par défaut en ms permettant de neutraliser les rebons dans Cevent et Cbutton
#define CLICK_TIME 500         //Valeur maxi en ms permettant de considerer un créneau comme un clic
//Class CButton
#define LONG_PRESS_TIME 2000  //Réglage de long press en ms pour Cbutton

#define LOG_TRANSITION true  //Permet de mettre un log lors du franchissement d'une transition

#define FILTERED_TIME 500  //Réglage de du filtarge en ms  pour CeventFiltered

//***********************************
//Stat
//***********************************
class Stat {
public:
  Stat(const char *stat_name = "", int stat_id = 0);  //Constructeur
  //Log
  bool log;       //Permet d'afficher les log (par défaut true)
  int id;         //Permet de donner un numéro d'identifiant à l'état
  String name() const;  //Renvoie le nom de l'état
  //Etat
  bool entry_() const;          //Vrai lorsque l'on rentre dans l'état
  bool do_() const;             //Vrai lorsque l'on est dans l'état
  bool exit_() const;           //Vrai lorsque l'on sort de l'état
  bool stat() const;            //stat = m_entry||m_do
  bool completion_event;  //à mettre à true lorsque l'état est consdéré comme terminé (permet de franchir une transition de type completion)

  //La fonction stat() permet de savoir si l'état est actif. Attention, elle ne doit jamais être utilisée dans la transition immédiatement suivante au risque de rendre le systeme instable.
  //En effet, dans ce cas l'activité m_entry risque de ne jamais être exécutée (Cas ou dans l'activité m_entry on inititalise une variable utilisée dans la transition suivant).
  //La fonction m_interne_stat() doit être utilisée à la place, _interne_stat() revoie true une fois m_entry exécutée et tant que l'on est pas en exit.

  bool interne_stat() const;  //interne_stat = m_do
  //Activation, Désactivation
  Stat *activate() ;    //Active l'état et renvoie un pointeur sur l'état courant, si p_active <> 0 alors l'état pointé est automatiquement désactivé (Sauf si p_active pointe vers lui même (cas d'un réactivation))
  void desactivate();  //Désactive l'état courant
  //After
  bool after(unsigned long delay_ms);  //Vaut true apres un délais imposée depuis l'activation de l'état ou le dernier resetStattime
  void resetStattime();                //Permet de remettre à la date courrante la date d'entrée dans l'état
  unsigned long getStattime();         //V1.5.5
  //Mise à jour
  bool update();  //A placer a la fin de la fonction loop OBLIGATOIREMENT
                  //Renvoie true si il y a une évolution et false sinon V1.4.3
private:
  //public:
  String m_name;  //Nom de l'état à afficher dans les logs
  //Variables carractérisant l'état
  bool m_entry;  //m_entry, m_do, m_exit ne peuvent pas être à true simultanément
  bool m_do;
  bool m_exit;
  unsigned long m_stattime;  //Date en ms de la dernière activation ou réactivation de l'état

  //Log
  void _log(String message, bool titre = false);    //Equivalent à print  si log=true (ne fait rien sinon), si titre = true alors on affiche le nom de la classe avant
  void _logln(String message, bool titre = false);  //Equivalent à println  si log=true (ne fait rien sinon), si titre = true alors on affiche le nom de la classe avant
};

boolean transition(Stat *&pActive,  Stat &from,  Stat &to, boolean event, boolean guard = true);      //event: évenement, guard : condition de garde, from : état avant, to : état après, pActive : pointeur sur l'état actif
boolean completion(Stat *&pActive,  Stat &from,  Stat &to, boolean guard = true);                     //guard : condition de garde, from : état avant, to : état après, pActive : pointeur sur l'état actif
boolean when(Stat *&pActive,  Stat &from,  Stat &to, boolean condition, boolean guard = true);        //condition: condition booleene de franchissement, from : état avant, to : état après, pActive : pointeur sur l'état actif
boolean after(Stat *&pActive,  Stat &from,  Stat &to, unsigned long delay_ms, boolean guard = true);  //delay: durée avant le changement d'état, guard : condition de garde, from : état avant, to : état après, pActive : pointeur sur l'état actif

//***********************************
//Timer
//***********************************
class Timer {
public:
  Timer();
  void start();                        //Pour lancer le timer ou le relancer
  void stop();                         //Pour stopper le timer
  bool after(unsigned long delay_ms);  //Vaut true apres un délais imposée
  bool stat();
private:
  bool m_stat;  //Start ou stop
  unsigned long m_stattime;
};

//***********************************
//Pulse
//***********************************
class Pulse {
public:
  Pulse();
  bool every(unsigned long delay_ms);  //Retourne une impulsion tous le x ms
private:
  bool m_every;  //Permet de gérer le basculement de every
  unsigned long m_stattime;
};


//***********************************
//Event
//***********************************
class Event {
public:
  Event(const char *name = "");
  bool log;                                                                      //Permet d'afficher les log (par défaut false)
  void setPin(uint8_t pin, bool interne_pull_up = false, bool reverse = false);  //Spécifie si on active ou non la résistance de pull up, si oui, stat, rising et falling sont automatiquement inversés //Reverse permet d'inverser stat, rising et falling V1.5.4
  bool rising() const;                                                                 //Front montant
  bool falling() const;                                                                //Front descendant
  bool change() const;                                                                 //Front montant ou descendant
  bool stat() const;                                                                   //Etat
  unsigned long anti_rebounce_time;                                              //Valeur en ms permettant de neutraliser les rebons (réglé par défaut à ANTI_REBOUNCE_TIME)
  bool update();                                                                 //Mise a jour de l'évenement. A placer a la fin de la fonction loop OBLIGATOIREMENT
                                                                                 //Renvoie true si il y a une évolution et false sinon V1.4.3
  unsigned long getStattime();
protected:
  String m_name;  //Nom de l'évenement à afficher dans les logs
  bool m_stat;
  bool m_new_stat;
  bool m_rising;
  bool m_falling;
  uint8_t m_pin;
  bool m_reverse;
  unsigned long m_stattime;
};

//***********************************
//Button
//***********************************
class Button : public Event {
public:
  Button(const char *name = "");
  void setLongPressTime(unsigned long delay_ms);  //2 secondes par défaut (cf constructeur)
  bool longPress() const;                               //Evenement appui long (Passe à true après un délais défini par setLongPressTime
  bool longPressFalling() const;                        //Front descendant d'un appui long
  bool click() const;                                   //Clic inférieur à 500 ms
  bool doubleClick() const;                             //Clic inférieur à 500 ms
  bool longPressStat() const;                           //Passe et reste à 1 pendant tout l'appui long
  unsigned long anti_rebounce_time;               //Valeur en ms permettant de neutraliser les rebons (réglé par défaut à ANTI_REBOUNCE_TIME)
  bool update();                                  //Mise a jour de l'évenement. A placer a la fin de la fonction loop OBLIGATOIREMENT
                                                  //Renvoie true si il y a une évolution et false sinon V1.4.3

private:
  bool m_longPress;
  bool m_longPressEvent;
  bool m_click;
  bool m_clickEvent;
  bool m_doubleClickEvent;
  unsigned long m_long_press_time;
};

//***********************************
//EventFiltered
//***********************************
//Fonctionne comme Event mais ignore les événements inférieurs à une certaine durée (0.5s par défaut)
//Le changement d'état est donc retardé d'autant

class EventFiltered : public Event {
public:
  EventFiltered(const char *name = "");
  void setFilteredTime(unsigned long delay_ms);  //0.5 secondes par défaut (cf constructeur)
  bool update();                                 //Mise a jour de l'évenement. A placer a la fin de la fonction loop OBLIGATOIREMENT
                                                 //Renvoie true si il y a une évolution et false sinon V1.4.3
private:
  unsigned long m_filtered_time;
  bool m_wait;
  unsigned long m_wait_time;
};

//***********************************
//Cbistable
//***********************************
class Bistable {
public:
  Bistable(const char *name1 = "", const char *name2 = "");
  bool log;  //Permet d'afficher les log (par défaut false)
  void setPin(uint8_t pin1, uint8_t pin2, bool reverse = false);
  void set1(bool stat);
  void set2(bool stat);
  bool stat1() const;  //Etat
  bool stat2() const;  //Etat
  void update() {
    return;
  };  //Mise a jour de l'objet (Ne fait rien)
private:
  uint8_t m_pin1;
  uint8_t m_pin2;
  bool m_stat1;
  bool m_stat2;
  bool m_reverse;  //Permet d'inverser la commande
  String m_name1;  //Nom à afficher dans les logs
  String m_name2;  //Nom à afficher dans les logs
};
//***********************************
//DigitalOut
//***********************************
class DigitalOut {
public:
  DigitalOut(const char *name = "");
  bool log;  //Permet d'afficher les log (par défaut false)
  void setPin(uint8_t pin, bool reverse = false);
  void on(unsigned long delay_on = 0);           //delay = 0 => ON permanent, delay = valeur (ms) => ON pendant valeur puis OFF
  void off(unsigned long delay_off = 0);         //delay = 0 => OFF permanent, delay = valeur (ms) => ON pendant valeur puis OFF
  void set(bool on_off);                         //mise à 1 ou à 0
  void toggle();                                 //Bascule l'état de la sortie
  void flash(unsigned long flash_period = 500);  //Permet de faire clignotter la led à la période demandé en ms)
  bool stat() const;                                   //Etat
  bool change() const;                                 //true si action on ou off sur le cycle ou si évolution interne (V1.6.3 )
  bool rising() const;                                 //Front montant (V1.6.3 )
  bool falling() const;                                //Front descendant (V1.6.3 )
  bool endOfTimer() const;                             //Fin du timer (V1.6.3)
  bool update();                                 //Mise a jour de l'objet //Renvoie true si il y a une évolution et false sinon V1.6.3

private:
  uint8_t m_pin;
  String m_name;  //Nom à afficher dans les logs
  bool m_stat;
  bool m_flash;                  //Permet de gérer le clignotement
  bool m_change;                 //true si action on ou off sur le cycle
  bool m_reverse;                //Permet d'inverser la commande
  bool m_end_of_timer;           //true si fin de timer On ou Off
  unsigned long m_flash_period;  //Période de clignotement
  unsigned long m_stattime;
  unsigned long m_delay;  //Mémorise la durée avant changement d'état
};

#endif