#include <Arduino.h>
#include "obj_stat.h"

//******************************************
//Portail GTI Version GTI1 du 08 06 2020
//A DUGAS / CPGE PT ATS - Lycee Livet
//Prévu pour fonctionner sur une carte MEGA 2560
//Dans le menu outil Outils :
// * Choisir Type de carte : Arduino Mega 2560
//Dans le moniteur série :
// * Choisir une vitesse de 9600 bauds
//******************************************

//******************************************
//Déclaration hardware
//******************************************
//Entrées
#define BP_OUVERTURE A6  //Module Grove
#define BP_FERMETURE A8  //Module Grove

#define FC_OUVERTURE 4  //24V via opto-coupleur
#define FC_FERMETURE 6  //24V via opto-coupleur

//Sorties
#define OUVERTURE 8   //24V via relais
#define FERMETURE 10  //24V via relais

//******************************************
//Déclaration des états
//******************************************
Stat E_ferme("Ferme", 100);
Stat E_en_ouverture("En cours d'ouverture", 110);
Stat E_ouvert("Ouvert", 120);
Stat E_en_fermeture("En cours de fermeture", 130);

Stat* pEtat_actif;  //pointeur vers l'état actif

//******************************************
//Déclaration des évenements
//******************************************
Button bp_ouverture("bp_ouverture");
Button bp_fermeture("bp_fermeture");

Event fc_ouverture("fc_ouverture");
Event fc_fermeture("fc_fermeture");

//******************************************
//Déclaration des sorties
//******************************************
Bistable moteur("Relais d'ouverture", "Relais de fermeture");

//******************************************
//Setup
//******************************************
void setup() {
  //******************************************
  //Setup hardware
  //******************************************
  //Entrées
  bp_ouverture.setPin(BP_OUVERTURE);
  bp_fermeture.setPin(BP_FERMETURE);
  fc_ouverture.setPin(FC_OUVERTURE);
  fc_fermeture.setPin(FC_FERMETURE);

  //Sorties
  moteur.setPin(OUVERTURE, FERMETURE);

  //******************************************
  //Port série
  //******************************************
  Serial.begin(9600);
  //******************************************
  //Initialisation des évenements
  //******************************************
  bp_ouverture.update();
  bp_fermeture.update();
  fc_ouverture.update();
  fc_fermeture.update();
  //******************************************
  //Etat initial
  //******************************************
  if (!fc_fermeture.stat())
    pEtat_actif = E_ouvert.activate();
  else
    pEtat_actif = E_ferme.activate();
  //******************************************
  //Activation des log
  //******************************************
  moteur.log = true;
  bp_ouverture.log = true;
  bp_fermeture.log = true;
  fc_ouverture.log = true;
  fc_fermeture.log = true;
}

//******************************************
//loop
//******************************************
void loop() {
  //******************************************
  //Evolution des états
  //******************************************

  //Ouvert->En Fermeture
  transition(pEtat_actif, E_ouvert, E_en_fermeture, bp_fermeture.rising());

  //En Fermeture->Fermé
  transition(pEtat_actif, E_en_fermeture, E_ferme, fc_fermeture.rising());

  //Fermé->En ouverture
  transition(pEtat_actif, E_ferme, E_en_ouverture, bp_ouverture.rising());

  //En ouverture->Ouvert
  transition(pEtat_actif, E_en_ouverture, E_ouvert, fc_ouverture.rising());

  //******************************************
  //Actions pendant les états
  //******************************************
  //----------------------------Exit-----------------------------------//

  //------------------------------Do-----------------------------------//

  //----------------------------Entry----------------------------------//
  if (E_en_ouverture.entry_()) { /*entry relatif à l'état en ouverture*/
    moteur.set1(HIGH);
  }
  if (E_en_fermeture.entry_()) { /*entry relatif à l'état en fermeture*/
    moteur.set2(HIGH);
  }
  if (E_ouvert.entry_()) { /*entry relatif à l'état ouvert*/
    moteur.set1(LOW);
  }
  if (E_ferme.entry_()) { /*exit relatif à l'état fermé*/
    moteur.set2(LOW);
  }

  //******************************************
  //Mise à jour des états
  //******************************************
  E_ouvert.update();
  E_ferme.update();
  E_en_ouverture.update();
  E_en_fermeture.update();
  //******************************************
  //Mise à jour des évenements
  //******************************************
  bp_ouverture.update();
  bp_fermeture.update();

  fc_ouverture.update();
  fc_fermeture.update();
}