#include <Arduino.h>
#include "obj_stat.h"

//******************************************
//Portail GTI Version GTI2 du 08 06 2020
//A DUGAS / CPGE PT ATS - Lycee Livet
//Prévu pour fonctionner sur une carte MEGA 2560
//Dans le menu outil Outils :
// * Choisir Type de carte : Arduino Mega 2560
//Dans le moniteur série :
// * Choisir une vitesse de 9600 bauds
//******************************************

//******************************************
//Déclaration hardware
//******************************************
//Entrées
#define BP_OUVERTURE A6  //Module Grove
#define BP_FERMETURE A8  //Module Grove

#define FC_OUVERTURE 4  //24V via opto-coupleur
#define FC_FERMETURE 6  //24V via opto-coupleur

//Sorties
#define OUVERTURE 8   //24V via relais
#define FERMETURE 10  //24V via relais

#define LED_MVT A0     //Module Grove (Rouge) /* Ajout, non présent dans GTI1*/
#define LED_OUVERT A2  //Module Grove (Verte) /* Ajout, non présent dans GTI1*/
#define LED_FERME A4   //Module Grove (Rouge) /* Ajout, non présent dans GTI1*/

//******************************************
//Déclaration des états
//******************************************
Stat E_ferme("Ferme", 100);
Stat E_en_ouverture("En cours d'ouverture", 110);
Stat E_ouvert("Ouvert", 120);
Stat E_en_fermeture("En cours de fermeture", 130);

Stat* pEtat_actif;  //pointeur vers l'état actif

//******************************************
//Déclaration des évenements
//******************************************
Button bp_ouverture("bp_ouverture");
Button bp_fermeture("bp_fermeture");
Event fc_ouverture("fc_ouverture");
Event fc_fermeture("fc_fermeture");

//******************************************
//Déclaration des sorties
//******************************************
Bistable moteur("Relais d'ouverture", "Relais de fermeture");

DigitalOut led_mvt("Led de d'avertissement");    /* Ajout, non présent dans GTI1*/
DigitalOut led_fc_ouverture("Led fc ouverture"); /* Ajout, non présent dans GTI1*/
DigitalOut led_fc_fermeture("Led fc fermeture"); /* Ajout, non présent dans GTI1*/

//******************************************
//Setup
//******************************************
void setup() {
  //******************************************
  //Setup hardware
  //******************************************
  //Entrées
  bp_ouverture.setPin(BP_OUVERTURE);
  bp_fermeture.setPin(BP_FERMETURE);
  fc_ouverture.setPin(FC_OUVERTURE);
  fc_fermeture.setPin(FC_FERMETURE);

  //Sorties
  led_mvt.setPin(LED_MVT);
  led_fc_ouverture.setPin(LED_OUVERT);
  led_fc_fermeture.setPin(LED_FERME);
  moteur.setPin(OUVERTURE, FERMETURE);

  //******************************************
  //Port série
  //******************************************
  Serial.begin(9600);
  //******************************************
  //Initialisation des évènements
  //******************************************
  bp_ouverture.update();
  bp_fermeture.update();

  fc_ouverture.update();
  fc_fermeture.update();
  //******************************************
  //Etat initial
  //******************************************
  if (!fc_fermeture.stat())
    pEtat_actif = E_ouvert.activate();
  else
    pEtat_actif = E_ferme.activate();
  //******************************************
  //Activation des log
  //******************************************
  led_mvt.log = true;
  led_fc_ouverture.log = true;
  led_fc_fermeture.log = true;
  moteur.log = true;
}
//******************************************
//loop
//******************************************
void loop() {
  //******************************************
  //Evolution des états
  //******************************************

  //Ouvert->En Fermeture
  transition(pEtat_actif, E_ouvert, E_en_fermeture, bp_fermeture.rising());

  //En Fermeture->Fermé
  transition(pEtat_actif, E_en_fermeture, E_ferme, fc_fermeture.rising());

  //Fermé->En ouverture
  transition(pEtat_actif, E_ferme, E_en_ouverture, bp_ouverture.rising());

  //En ouverture->Ouvert
  transition(pEtat_actif, E_en_ouverture, E_ouvert, fc_ouverture.rising());

  //******************************************
  //Actions pendant les états
  //******************************************

  //----------------------------Exit-----------------------------------//
  if (E_ouvert.exit_()) { /*exit relatif à l'état ouvert*/
    //Zone à complèter
    //...
    led_fc_ouverture.off();
  }
  if (E_ferme.exit_()) { /*exit relatif à l'état fermé*/
    //Zone à complèter
    //...
    led_fc_fermeture.off();
  }
  //------------------------------Do------------------------------------//
  if (E_en_ouverture.do_()) { /*do relatif à l'état en ouverture*/
                              //Zone à complèter
    //...
    led_mvt.flash(500);
  }
  if (E_en_fermeture.do_()) { /*do relatif à l'état en fermeture*/
                              //Zone à complèter
    //...
    led_mvt.flash(500);
  }
  //----------------------------Entry------------------------------------//
  if (E_en_ouverture.entry_()) { /*entry relatif à l'état en ouverture*/
    moteur.set1(HIGH);
  }
  if (E_en_fermeture.entry_()) { /*entry relatif à l'état en fermeture*/
    moteur.set2(HIGH);
  }
  if (E_ouvert.entry_()) { /*entry relatif à l'état ouvert*/
    moteur.set1(LOW);
    //Zone à complèter
    //...
    led_fc_ouverture.on();
    led_mvt.off();
  }
  if (E_ferme.entry_()) { /*exit relatif à l'état fermé*/
    moteur.set2(LOW);
    //Zone à complèter
    //...
    led_fc_fermeture.on();
    led_mvt.off();
  }

  //******************************************
  //Fin Actions pendant les états
  //******************************************

  //******************************************
  //Mise à jour des états
  //******************************************
  E_ouvert.update();
  E_ferme.update();
  E_en_ouverture.update();
  E_en_fermeture.update();

  //******************************************
  //Mise à jour des évenements
  //******************************************
  bp_ouverture.update();
  bp_fermeture.update();
  fc_ouverture.update();
  fc_fermeture.update();
  //******************************************
  //Mise à jour des sorties
  //******************************************
  led_mvt.update();          /* Ajout, non présent dans GTI1*/
  led_fc_ouverture.update(); /* Ajout, non présent dans GTI1*/
  led_fc_fermeture.update(); /* Ajout, non présent dans GTI1*/
}